from Crypto.Cipher import AES
from base64 import b64decode, b64encode


class aes128cbc(object):

    _key = None
    _iv = None

    def __init__(self, key, iv):
        """
        Advanced Encryption Standard object
        """
        self._bs = 16  # Block size

        if key is None:
            raise ValueError('Missing AES key')
        else:
            self._key = key

        if iv is None:
            raise ValueError('Missing AES initialization vector')
        else:
            self._iv = iv

    def encrypt(self, msg):
        msg = self._pad(self.str_to_bytes(msg))
        crypt = AES.new(self._key, AES.MODE_CBC, self._iv)
        return b64encode(crypt.encrypt(msg)).decode('utf-8')

    def decrypt(self, msg):
        msg = b64decode(msg)
        crypt = AES.new(self._key, AES.MODE_CBC, self._iv)
        return self._unpad(crypt.decrypt(msg)).decode('utf-8')

    def _pad(self, s):
        return s + (self._bs - len(s) % self._bs) * self.str_to_bytes(chr(
            self._bs - len(s) % self._bs))

    def _unpad(self, s):
        return s[:-ord(s[len(s)-1:])]

    def str_to_bytes(self, s):
        u_type = type(b''.decode('utf8'))
        if isinstance(s, u_type):
            return s.encode('utf8')
        return s



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from oauthlib.oauth1.rfc5849.request_validator import RequestValidator
from oauthlib.oauth1.rfc5849.utils import UNICODE_ASCII_CHARACTER_SET
from blti import BLTIException
import time
import re


class BLTIRequestValidator(RequestValidator):
    @property
    def allowed_signature_methods(self):
        return ['HMAC-SHA1']

    @property
    def dummy_client(self):
        return 'dummy'

    @property
    def client_key_length(self):
        return 12, 30

    @property
    def nonce_length(self):
        return 20, 50

    @property
    def enforce_ssl(self):
        return getattr(
            settings, 'LTI_ENFORCE_SSL', True)

    @property
    def safe_characters(self):
        return set(UNICODE_ASCII_CHARACTER_SET) | set('-_')

    def validate_client_key(self, client_key, request):
        return self.get_client_secret(
            client_key, request) != self.dummy_client

    def get_client_secret(self, client_key, request):
        try:
            return getattr(
                settings, 'LTI_CONSUMERS', {})[client_key]
        except KeyError:
            return self.dummy_client

    def validate_timestamp_and_nonce(self, client_key, timestamp, nonce,
                                     request, request_token=None,
                                     access_token=None):
        now = int(time.time())
        return (now - 60) <= int(timestamp) <= (now + 60)


class Roles(object):
    # https://www.imsglobal.org/specs/ltiv1p1/implementation-guide#toc-30
    CANVAS_ROLES = {
        'member': ['Administrator', 'Instructor', 'TeachingAssistant',
                   'ContentDeveloper', 'Learner', 'Observer'],
        'admin': ['Administrator', 'Instructor', 'TeachingAssistant',
                  'ContentDeveloper'],
    }

    RE_ROLE_NS = re.compile(r'^urn:lti:(?:inst|sys)?role:ims/lis/([A-Za-z]+)$')

    def authorize(self, blti, role='member'):
        if blti is None:
            raise BLTIException('Missing LTI parameters')

        lti_consumer = blti.data.get(
            'tool_consumer_info_product_family_code', '').lower()

        if lti_consumer == 'canvas':
            if (not role or role == 'public'):
                pass
            elif (role in self.CANVAS_ROLES):  # member/admin
                self._has_role(blti, self.CANVAS_ROLES[role])
            else:  # specific role?
                self._has_role(blti, [role])
        else:
            raise BLTIException('authorize() not implemented for "%s"!' % (
                lti_consumer))

    def _has_role(self, blti, valid_roles):
        roles = blti.data.get('roles', '').split(',')
        for role in roles:
            if role in valid_roles:
                return

            m = self.RE_ROLE_NS.match(role)
            if m and m.group(1) in valid_roles:
                return

        raise BLTIException('You are not authorized to view this content')



import json
from base64 import b64decode, b64encode
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from blti.crypto import aes128cbc


class BLTIException(Exception):
    pass


class BLTI(object):
    def __init__(self):
        if not hasattr(settings, 'BLTI_AES_KEY'):
            raise ImproperlyConfigured('Missing setting BLTI_AES_KEY')
        if not hasattr(settings, 'BLTI_AES_IV'):
            raise ImproperlyConfigured('Missing setting BLTI_AES_IV')

    def set_session(self, request, **kwargs):
        if not request.session.exists(request.session.session_key):
            request.session.create()

        kwargs['_blti_session_id'] = request.session.session_key
        request.session['blti'] = self._encrypt_session(
            self.filter_oauth_params(kwargs))

    def get_session(self, request):
        if 'blti' not in request.session:
            raise BLTIException('Invalid Session')

        blti_data = self._decrypt_session(request.session['blti'])
        if blti_data['_blti_session_id'] != request.session.session_key:
            raise BLTIException('Invalid BLTI session data')

        blti_data.pop('_blti_session_id', None)
        return blti_data

    def filter_oauth_params(self, params):
        return {k: v for k, v in params.items() if not k.startswith('oauth_')}

    def _encrypt_session(self, data):
        aes = aes128cbc(settings.BLTI_AES_KEY, settings.BLTI_AES_IV)
        return aes.encrypt(json.dumps(data))

    def _decrypt_session(self, string):
        aes = aes128cbc(settings.BLTI_AES_KEY, settings.BLTI_AES_IV)
        return json.loads(aes.decrypt(string))



"""
Cross Site Request Forgery and Session HTTP Header Middleware.

This module provides middleware that implements protection
against request forgeries from other sites.
"""

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin


class CSRFHeaderMiddleware(MiddlewareMixin):
    def process_request(self, request):
        csrf_token = request.META.get('HTTP_X_CSRFTOKEN', None)
        if csrf_token is not None:
            csrf_token_name = settings.CSRF_COOKIE_NAME
            request.COOKIES[csrf_token_name] = csrf_token


class SessionHeaderMiddleware(MiddlewareMixin):
    def process_request(self, request):
        session_id = request.META.get('HTTP_X_SESSIONID', None)
        if session_id is not None:
            session_key = settings.SESSION_COOKIE_NAME
            request.COOKIES[session_key] = session_id


class SameSiteMiddleware(MiddlewareMixin):
    def process_response(self, request, response):
        if 'sessionid' in response.cookies:
            response.cookies['sessionid']['samesite'] = 'None'
        if 'csrftoken' in response.cookies:
            response.cookies['csrftoken']['samesite'] = 'None'
        return response
